@@ -3,7 +3,8 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -507,13 +508,6 @@ USA.
 (define (fits-in-unsigned-48? x)
   (<= 0 x #xffffffffffff))
 
-;; XXX doesn't belong here
-
-(define-integrable type-code:fixnum #x1a)
-(define-integrable type-code:flonum #x06)
-(define-integrable type-code:manifest-closure #x0d)
-(define-integrable type-code:manifest-vector #x00)
-
 ;; XXX
 
 (define (non-pointer->literal object)
@@ -534,7 +528,6 @@ USA.
   '(
     &/                  ;nobody open-codes this
     DIVIDE-FIXNUM       ;nobody open-codes this
-    FIXNUM-LSH          ;open-coding not useful without constant operands
     FLOATING-VECTOR-CONS;nobody open-codes this
     FLONUM-ACOS         ;not useful to open-code hairy math
     FLONUM-ASIN         ;not useful to open-code hairy math
