@@ -3,7 +3,8 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -629,7 +630,8 @@ TRANSCRIPT    messages appear in transcript buffer, if it is enabled;
 			 (next (cdr threads)))
 		     (let ((thread (weak-car record))
 			   (flags (weak-cdr record)))
-		       (if (and thread (not (thread-dead? thread)))
+		       (if (and (thread? thread)
+				(not (thread-dead? thread)))
 			   (loop next
 				 threads
 				 (if (car flags)
