@@ -1,6 +1,6 @@
 #| -*-Scheme-*- |#
 
-(define-load-option 'GDBM
+(define-load-option 'gdbm
   (standard-system-loader "."))
 
 (further-load-options #t)
\ No newline at end of file
