@@ -3,7 +3,8 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -412,6 +413,12 @@ USA.
 	(lambda ()
 	  (eval file-object (package/environment package))))))
 
+  (let ((package (find-package '(runtime equality))))
+    (with-current-package package
+      (lambda ()
+	((lexical-reference (package/environment package)
+			    'initialize-package!)))))
+
   (load-files-with-boot-inits files1)
   (call-pkg-init-proc '(runtime gc-daemons) 'initialize-package!)
   (call-pkg-init-proc '(runtime garbage-collector) 'initialize-package!)
