@@ -3,7 +3,8 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019, 2020 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -283,7 +284,7 @@ USA.
       (guarantee regsexp-match? match 'regsexp-replacer)
       (let ((builder (string-builder)))
 	(replacer builder match)
-	(builder)))))
+	(builder 'immutable)))))
 
 (define (%regsexp-replacer replacement caller)
   (let loop ((r replacement))
@@ -344,13 +345,13 @@ USA.
                   (builder (car exprs))
                   (optimize-alt (cdr exprs) builder))
                 (if builder
-                    (cons (builder)
+                    (cons (builder 'immutable)
                           (cons (car exprs)
                                 (optimize-alt (cdr exprs) #f)))
                     (cons (car exprs)
                           (optimize-alt (cdr exprs) #f))))
             (if builder
-                (list (builder))
+                (list (builder 'immutable))
                 '())))
 
       (define (dispatch)
