@@ -3,7 +3,8 @@
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
     2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
-    2017, 2018, 2019 Massachusetts Institute of Technology
+    2017, 2018, 2019, 2020, 2021, 2022 Massachusetts Institute of
+    Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -29,29 +30,24 @@ USA.
 
 (declare (usual-integrations))
 
-(define (initialize-package!)
-  (set! working-directory-pathname
-	(make-general-parameter #f
-				default-parameter-converter
-				default-parameter-merger
-				default-parameter-getter
-				wd-setter))
-  (reset!)
-  (add-event-receiver! event:after-restore reset!))
-
-(define working-directory-pathname)
+(define-deferred *working-directory-pathname*
+  (make-general-parameter #f
+			  default-parameter-converter
+			  default-parameter-merger
+			  default-parameter-getter
+			  wd-setter))
 
 (define (wd-setter set-param pathname)
   (set-param pathname)
-  (param:default-pathname-defaults pathname)
   pathname)
 
-(define (reset!)
-  (working-directory-pathname
-   (pathname-simplify
-    (pathname-as-directory
-     (string-from-primitive
-      ((ucode-primitive working-directory-pathname)))))))
+(define (working-directory-pathname)
+  (or (*working-directory-pathname*)
+      (pathname-simplify
+       (pathname-as-directory
+	(parse-namestring
+	 ((ucode-primitive working-directory-pathname))
+	 local-host)))))
 
 (define (set-working-directory-pathname! name)
   (let ((pathname (new-pathname name)))
@@ -69,16 +65,14 @@ USA.
 				  "no such directory")
 			      'set-working-directory-pathname!
 			      (list name)))
-    (working-directory-pathname pathname)
+    (*working-directory-pathname* pathname)
     (cmdl/set-default-directory (nearest-cmdl) pathname)
     pathname))
 
 (define (with-working-directory-pathname name thunk)
   (let ((pathname (new-pathname name)))
-    (fluid-let ((*default-pathname-defaults* pathname))
-      (parameterize ((param:default-pathname-defaults pathname)
-		     (working-directory-pathname pathname))
-	(thunk)))))
+    (parameterize ((*working-directory-pathname* pathname))
+      (thunk))))
 
 (define (new-pathname name)
   (pathname-simplify
